@@ -28,11 +28,15 @@
 #include <math.h>
 
 
+
 #include "speak_lib.h"
 #include "speech.h"
 #include "phoneme.h"
 #include "synthesize.h"
 #include "voice.h"
+#include "sonic.h"
+
+//#undef INCLUDE_KLATT
 
 #ifdef USE_PORTAUDIO
 #include "portaudio.h"
@@ -51,14 +55,13 @@
 
 #define PI  3.1415927
 #define PI2 6.283185307
-#define STEPSIZE  64                // 2.9mS at 22 kHz sample rate
 #define N_WAV_BUF   10
 
-static voice_t *wvoice;
+voice_t *wvoice;
 
 FILE *f_log = NULL;
 int option_waveout = 0;
-int option_harmonic1 = 11;   // 10
+static int option_harmonic1 = 10;   // 10
 int option_log_frames = 0;
 static int flutter_amp = 64;
 
@@ -77,15 +80,14 @@ static wavegen_peaks_t peaks[N_PEAKS];
 static int peak_harmonic[N_PEAKS];
 static int peak_height[N_PEAKS];
 
-#define N_ECHO_BUF 5500   // max of 250mS at 22050 Hz
-static int echo_head;
-static int echo_tail;
+int echo_head;
+int echo_tail;
+int echo_amp = 0;
+short echo_buf[N_ECHO_BUF];
 static int echo_length = 0;   // period (in sample\) to ensure completion of echo at the end of speech, set in WavegenSetEcho()
-static int echo_amp = 0;
-static short echo_buf[N_ECHO_BUF];
 
 static int voicing;
-RESONATOR rbreath[N_PEAKS];
+static RESONATOR rbreath[N_PEAKS];
 
 static int harm_sqrt_n = 0;
 
@@ -98,24 +100,13 @@ static int hspect[2][MAX_HARMONIC];         // 2 copies, we interpolate between
 static int max_hval=0;
 
 static int nsamples=0;       // number to do
-static int amplitude = 32;
-static int amplitude_v = 0;
 static int modulation_type = 0;
 static int glottal_flag = 0;
 static int glottal_reduce = 0;
 
-static unsigned char *mix_wavefile = NULL;  // wave file to be added to synthesis
-static int n_mix_wavefile = 0;       // length in bytes
-static int mix_wave_scale = 0;         // 0=2 byte samples
-static int mix_wave_amp = 32;
-static int mix_wavefile_ix = 0;
 
-static int pitch;          // pitch Hz*256
-static int pitch_ix;       // index into pitch envelope (*256)
-static int pitch_inc;      // increment to pitch_ix
-static unsigned char *pitch_env=NULL;
-static int pitch_base;     // Hz*256 low, before modified by envelope
-static int pitch_range;    // Hz*256 range of envelope
+WGEN_DATA wdata;
+
 static int amp_ix;
 static int amp_inc;
 static unsigned char *amplitude_env = NULL;
@@ -144,8 +135,8 @@ int wcmdq_head=0;
 int wcmdq_tail=0;
 
 // pitch,speed,
-int embedded_default[N_EMBEDDED_VALUES]        = {0,50,170,100,50, 0,0, 0,170,0,0,0,0,0};
-static int embedded_max[N_EMBEDDED_VALUES]     = {0,0x7fff,400,300,99,99,99, 0,360,0,0,0,0,4};
+int embedded_default[N_EMBEDDED_VALUES]        = {0,    50,175,100,50, 0, 0, 0,175,0,0,0,0,0,0};
+static int embedded_max[N_EMBEDDED_VALUES]     = {0,0x7fff,750,300,99,99,99, 0,750,0,0,0,0,4,0};
 
 #define N_CALLBACK_IX N_WAV_BUF-2   // adjust this delay to match display with the currently spoken word
 int current_source_index=0;
@@ -159,35 +150,8 @@ static PortAudioStream *pa_stream=NULL;
 static PaStream *pa_stream=NULL;
 #endif
 
-/* default pitch envelope, a steady fall */
-#define ENV_LEN  128
-
-
-/*
-unsigned char Pitch_env0[ENV_LEN] = {
-    255,253,251,249,247,245,243,241,239,237,235,233,231,229,227,225,
-    223,221,219,217,215,213,211,209,207,205,203,201,199,197,195,193,
-    191,189,187,185,183,181,179,177,175,173,171,169,167,165,163,161,
-    159,157,155,153,151,149,147,145,143,141,139,137,135,133,131,129,
-    127,125,123,121,119,117,115,113,111,109,107,105,103,101, 99, 97,
-     95, 93, 91, 89, 87, 85, 83, 81, 79, 77, 75, 73, 71, 69, 67, 65,
-     63, 61, 59, 57, 55, 53, 51, 49, 47, 45, 43, 41, 39, 37, 35, 33,
-     31, 29, 27, 25, 23, 21, 19, 17, 15, 13, 11,  9,  7,  5,  3,  1
-};
-*/
-
-/*
-unsigned char Pitch_long[ENV_LEN] = {
-	254,249,250,251,252,253,254,254, 255,255,255,255,254,254,253,252,
-	251,250,249,247,244,242,238,234, 230,225,221,217,213,209,206,203,
-	199,195,191,187,183,179,175,172, 168,165,162,159,156,153,150,148,
-	145,143,140,138,136,134,132,130, 128,126,123,120,117,114,111,107,
-	104,100,96,91, 86,82,77,73, 70,66,63,60, 58,55,53,51,
-	49,47,46,45, 43,42,40,38, 36,34,31,28, 26,24,22,20,
-	18,16,14,12, 11,10,9,8, 8,8,8,8, 9,8,8,8,
-	8,8,7,7, 6,6,6,5, 4,4,3,3, 2,1,1,0
-};
-*/
+static sonicStream sonicSpeedupStream = NULL;
+double sonicSpeed = 1.0;
 
 // 1st index=roughness
 // 2nd index=modulation_type
@@ -262,7 +226,7 @@ static const unsigned char Flutter_tab[N_FLUTTER] = {
 };
 
 // waveform shape table for HF peaks, formants 6,7,8
-#define N_WAVEMULT 512
+#define N_WAVEMULT 128
 static int wavemult_offset=0;
 static int wavemult_max=0;
 
@@ -276,32 +240,8 @@ static unsigned char wavemult[N_WAVEMULT] = {
    218,213,207,201,194,188,181,174,166,159,152,144,136,128,121,113,
    105, 98, 90, 83, 76, 69, 62, 55, 49, 43, 37, 32, 27, 22, 18, 14,
     11,  8,  5,  3,  2,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
-     0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
      0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0 };
-   
+ 
 
 // set from y = pow(2,x) * 128,  x=-1 to 1
 unsigned char pitch_adjust_tab[MAX_PITCH_VALUE+1] = {
@@ -319,8 +259,6 @@ unsigned char pitch_adjust_tab[MAX_PITCH_VALUE+1] = {
    217,220,223,226,229,232,236,239,
    242,246,249,252, 254,255 };
 
-int WavegenFill(int fill_zeros);
-
 
 #ifdef LOG_FRAMES
 static void LogMarker(int type, int value)
@@ -348,9 +286,16 @@ void WcmdqStop()
 {//=============
 	wcmdq_head = 0;
 	wcmdq_tail = 0;
+	if(sonicSpeedupStream != NULL)
+	{
+		sonicDestroyStream(sonicSpeedupStream);
+		sonicSpeedupStream = NULL;
+	}
 #ifdef USE_PORTAUDIO
 	Pa_AbortStream(pa_stream);
 #endif
+	if(mbrola_name[0] != 0)
+		MbrolaReset();
 }
 
 
@@ -385,8 +330,8 @@ static void WcmdqIncHead()
 // data points from which to make the presets for pk_shape1 and pk_shape2
 #define PEAKSHAPEW 256
 static const float pk_shape_x[2][8] = {
-	{0,-0.6, 0.0, 0.6, 1.4, 2.5, 4.5, 5.5},
-	{0,-0.6, 0.0, 0.6, 1.4, 2.0, 4.5, 5.5 }};
+	{0,-0.6f, 0.0f, 0.6f, 1.4f, 2.5f, 4.5f, 5.5f},
+	{0,-0.6f, 0.0f, 0.6f, 1.4f, 2.0f, 4.5f, 5.5f }};
 static const float pk_shape_y[2][8] = {
 	{0,  67,  81,  67,  31,  14,   0,  -6} ,
 	{0,  77,  81,  77,  31,   7,   0,  -6 }};
@@ -474,6 +419,10 @@ static int userdata[4];
 static PaError pa_init_err=0;
 static int out_channels=1;
 
+unsigned char *outbuffer = NULL;
+int outbuffer_size = 0;
+
+
 #if USE_PORTAUDIO == 18
 static int WaveCallback(void *inputBuffer, void *outputBuffer,
 		unsigned long framesPerBuffer, PaTimestamp outTime, void *userData )
@@ -486,9 +435,30 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	int ix;
 	int result;
 	unsigned char *p;
+	unsigned char *out_buf;
+	unsigned char *out_end2;
+	int pa_size;
 
-	out_ptr = out_start = (unsigned char *)outputBuffer;
-	out_end = out_ptr + framesPerBuffer*2;
+	pa_size = framesPerBuffer*2;
+
+	// make a buffer 3x size of the portaudio output
+	ix = pa_size*3;
+	if(ix > outbuffer_size)
+	{
+		outbuffer = (unsigned char *)realloc(outbuffer, ix);
+		if(outbuffer == NULL)
+		{
+			fprintf(stderr, "espeak: out of memory\n");
+		}
+		outbuffer_size = ix;
+		out_ptr = NULL;
+	}
+	if(out_ptr == NULL)
+	{
+		out_ptr = out_start = outbuffer;
+		out_end = out_start + outbuffer_size;
+	}
+	out_end2 = &outbuffer[pa_size];  // top of data needed for the portaudio buffer
 
 #ifdef LIBRARY
 	event_list_ix = 0;
@@ -496,6 +466,24 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 
 	result = WavegenFill(1);
 
+	// copy from the outbut buffer into the portaudio buffer
+	if(result && (out_ptr > out_end2))
+	{
+		result = 0;   // don't end yet, there is more data in the buffer than can fit in portaudio
+	}
+
+	while(out_ptr < out_end2)
+		*out_ptr++ = 0;  // fill with zeros up to the size of the portaudio buffer
+
+	memcpy(outputBuffer, outbuffer, pa_size);
+
+	// move the remaining contents of the start of the output buffer
+	for(p = out_end2; p < out_end; p++)
+	{
+		p[-pa_size] = p[0];
+	}
+	out_ptr -= pa_size;
+
 #ifdef LIBRARY
 	count_samples += framesPerBuffer;
 	if(synth_callback)
@@ -516,14 +504,15 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	{
 		// swap the order of bytes in each sound sample in the portaudio buffer
 		int c;
-		out_ptr = (unsigned char *)outputBuffer;
-		out_end = out_ptr + framesPerBuffer*2;
-		while(out_ptr < out_end)
+		unsigned char *buf_end;
+		out_buf = (unsigned char *)outputBuffer;
+		buf_end = out_buf + framesPerBuffer*2;
+		while(out_buf < buf_end)
 		{
-			c = out_ptr[0];
-			out_ptr[0] = out_ptr[1];
-			out_ptr[1] = c;
-			out_ptr += 2;
+			c = out_buf[0];
+			out_buf[0] = out_buf[1];
+			out_buf[1] = c;
+			out_buf += 2;
 		}
 	}
 #endif
@@ -532,12 +521,12 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 	{
 		// sound output can only do stereo, not mono.  Duplicate each sound sample to
 		// produce 2 channels.
-		out_ptr = (unsigned char *)outputBuffer;
+		out_buf = (unsigned char *)outputBuffer;
 		for(ix=framesPerBuffer-1; ix>=0; ix--)
 		{
-			p = &out_ptr[ix*4];
-			p[3] = p[1] = out_ptr[ix*2 + 1];
-			p[2] = p[0] = out_ptr[ix*2];
+			p = &out_buf[ix*4];
+			p[3] = p[1] = out_buf[ix*2 + 1];
+			p[2] = p[0] = out_buf[ix*2];
 		}
 	}
 
@@ -569,7 +558,7 @@ static int WaveCallback(const void *inputBuffer, void *outputBuffer,
 /* This is a fixed version of Pa_OpenDefaultStream() for use if the version in portaudio V19
    is broken */
 
-PaError Pa_OpenDefaultStream2( PaStream** stream,
+static PaError Pa_OpenDefaultStream2( PaStream** stream,
                               int inputChannelCount,
                               int outputChannelCount,
                               PaSampleFormat sampleFormat,
@@ -672,24 +661,31 @@ int WavegenOpenSound()
 }
 
 
+
 int WavegenCloseSound()
 {//====================
 	PaError active;
 
 	// check whether speaking has finished, and close the stream
-//	if(pa_stream != NULL)
-	if((pa_stream != NULL) && (WcmdqUsed() == 0))  // TEST, also check that queue is empty
+	if(pa_stream != NULL)
 	{
 #if USE_PORTAUDIO == 18
 		active = Pa_StreamActive(pa_stream);
 #else
 		active = Pa_IsStreamActive(pa_stream);
 #endif
-		if(active == 0)
+		if(WcmdqUsed() == 0)   // also check that the queue is empty
 		{
-			Pa_CloseStream(pa_stream);
-			pa_stream = NULL;
-			return(1);
+			if(active == 0)
+			{
+				Pa_CloseStream(pa_stream);
+				pa_stream = NULL;
+				return(1);
+			}
+		}
+		else
+		{
+			WavegenOpenSound();  // still items in the queue, shouldn't be closed
 		}
 	}
 	return(0);
@@ -737,6 +733,7 @@ void WavegenInit(int rate, int wavemult_fact)
 	if(wavemult_fact == 0)
 		wavemult_fact=60;  // default
 
+	wvoice = NULL;
 	samplerate = samplerate_native = rate;
 	PHASE_INC_FACTOR = 0x8000000 / samplerate;   // assumes pitch is Hz*32
 	Flutter_inc = (64 * samplerate)/rate;
@@ -745,6 +742,9 @@ void WavegenInit(int rate, int wavemult_fact)
 	wavephase = 0x7fffffff;
 	max_hval = 0;
 
+	wdata.amplitude = 32;
+	wdata.amplitude_fmt = 100;
+
 	for(ix=0; ix<N_EMBEDDED_VALUES; ix++)
 		embedded_value[ix] = embedded_default[ix];
 
@@ -769,10 +769,15 @@ void WavegenInit(int rate, int wavemult_fact)
 
 	WavegenInitPkData(1);
 	WavegenInitPkData(0);
-	pk_shape = pk_shape2;         // ph_shape2
+	pk_shape = pk_shape2;         // pk_shape2
+
+#ifdef INCLUDE_KLATT
+	KlattInit();
+#endif
 
 #ifdef LOG_FRAMES
 remove("log-espeakedit");
+remove("log-klatt");
 #endif
 }  // end of WavegenInit
 
@@ -784,7 +789,7 @@ int GetAmplitude(void)
 	// normal, none, reduced, moderate, strong
 	static const unsigned char amp_emphasis[5] = {16, 16, 10, 16, 22};
 
-	amp = (embedded_value[EMBED_A])*60/100;
+	amp = (embedded_value[EMBED_A])*55/100;
 	general_amplitude = amp * amp_emphasis[embedded_value[EMBED_F]] / 16;
 	return(general_amplitude);
 }
@@ -813,12 +818,15 @@ static void WavegenSetEcho(void)
 		amp = embedded_value[EMBED_H];
 		delay = 130;
 	}
+#ifdef deleted
 	if(embedded_value[EMBED_T] > 0)
 	{
-		// announcing punctuation
+		// announcing punctuation, add a small echo
+// This seems unpopular
 		amp = embedded_value[EMBED_T] * 8;
 		delay = 60;
 	}
+#endif
 
 	if(delay == 0)
 		amp = 0;
@@ -903,6 +911,24 @@ int PeaksToHarmspect(wavegen_peaks_t *peaks, int pitch, int *htab, int control)
 		}
 	}
 
+{
+int y;
+int h2;
+	// increase bass
+	y = peaks[1].height * 10;   // addition as a multiple of 1/256s
+	h2 = (1000<<16)/pitch;       // decrease until 1000Hz
+	if(h2 > 0)
+	{
+		x = y/h2;
+		h = 1;
+		while(y > 0)
+		{
+			htab[h++] += y;
+			y -= x;
+		}
+	}
+}
+
 	// find the nearest harmonic for HF peaks where we don't use shape
 	for(; pk<N_PEAKS; pk++)
 	{
@@ -961,10 +987,10 @@ static void AdvanceParameters()
 	static int Flutter_ix = 0;
 
 	// advance the pitch
-	pitch_ix += pitch_inc;
-	if((ix = pitch_ix>>8) > 127) ix = 127;
-	x = pitch_env[ix] * pitch_range;
-	pitch = (x>>8) + pitch_base;
+	wdata.pitch_ix += wdata.pitch_inc;
+	if((ix = wdata.pitch_ix>>8) > 127) ix = 127;
+	x = wdata.pitch_env[ix] * wdata.pitch_range;
+	wdata.pitch = (x>>8) + wdata.pitch_base;
 
 	amp_ix += amp_inc;
 
@@ -973,9 +999,9 @@ static void AdvanceParameters()
 		Flutter_ix = 0;
 	x = ((int)(Flutter_tab[Flutter_ix >> 6])-0x80) * flutter_amp;
 	Flutter_ix += Flutter_inc;
-	pitch += x;
-	if(pitch < 102400)
-		pitch = 102400;   // min pitch, 25 Hz  (25 << 12)
+	wdata.pitch += x;
+	if(wdata.pitch < 102400)
+		wdata.pitch = 102400;   // min pitch, 25 Hz  (25 << 12)
 
 	if(samplecount == samplecount_start)
 		return;
@@ -989,14 +1015,24 @@ static void AdvanceParameters()
 			peaks[ix].height = 0;
 		peaks[ix].left1 += peaks[ix].left_inc;
 		peaks[ix].left = int(peaks[ix].left1);
-		peaks[ix].right1 += peaks[ix].right_inc;
-		peaks[ix].right = int(peaks[ix].right1);
+		if(ix < 3)
+		{
+			peaks[ix].right1 += peaks[ix].right_inc;
+			peaks[ix].right = int(peaks[ix].right1);
+		}
+		else
+		{
+			peaks[ix].right = peaks[ix].left;
+		}
 	}
-	for(;ix < N_PEAKS; ix++)
+	for(;ix < 8; ix++)
 	{
 		// formants 6,7,8 don't have a width parameter
-		peaks[ix].freq1 += peaks[ix].freq_inc;
-		peaks[ix].freq = int(peaks[ix].freq1);
+		if(ix < 7)
+		{
+			peaks[ix].freq1 += peaks[ix].freq_inc;
+			peaks[ix].freq = int(peaks[ix].freq1);
+		}
 		peaks[ix].height1 += peaks[ix].height_inc;
 		if((peaks[ix].height = int(peaks[ix].height1)) < 0)
 			peaks[ix].height = 0;
@@ -1077,8 +1113,8 @@ void InitBreath(void)
 
 
 
-void SetBreath()
-{//=============
+static void SetBreath()
+{//====================
 #ifndef PLATFORM_RISCOS
 	int pk;
 
@@ -1098,8 +1134,8 @@ void SetBreath()
 }  // end of SetBreath
 
 
-int ApplyBreath(void)
-{//==================
+static int ApplyBreath(void)
+{//=========================
 	int value = 0;
 #ifndef PLATFORM_RISCOS
 	int noise;
@@ -1159,26 +1195,27 @@ int Wavegen()
 			{
 				hswitch = 0;
 				harmspect = hspect[0];
-				maxh2 = PeaksToHarmspect(peaks,pitch<<4,hspect[0],0);
+				maxh2 = PeaksToHarmspect(peaks, wdata.pitch<<4, hspect[0], 0);
 
 				// adjust amplitude to compensate for fewer harmonics at higher pitch
-				amplitude2 = (amplitude * pitch)/(100 << 11);
+//				amplitude2 = (wdata.amplitude * wdata.pitch)/(100 << 11);
+				amplitude2 = (wdata.amplitude * (wdata.pitch >> 8) * wdata.amplitude_fmt)/(10000 << 3);
 
             // switch sign of harmonics above about 900Hz, to reduce max peak amplitude
-				h_switch_sign = 890 / (pitch >> 12);
+				h_switch_sign = 890 / (wdata.pitch >> 12);
 			}
 			else
 				AdvanceParameters();
 
 			// pitch is Hz<<12
-			phaseinc = (pitch>>7) * PHASE_INC_FACTOR;
-			cycle_samples = samplerate/(pitch >> 12);  // sr/(pitch*2)
-			hf_factor = pitch >> 11;
+			phaseinc = (wdata.pitch>>7) * PHASE_INC_FACTOR;
+			cycle_samples = samplerate/(wdata.pitch >> 12);  // sr/(pitch*2)
+			hf_factor = wdata.pitch >> 11;
 
 			maxh = maxh2;
 			harmspect = hspect[hswitch];
 			hswitch ^= 1;
-			maxh2 = PeaksToHarmspect(peaks,pitch<<4,hspect[hswitch],1);
+			maxh2 = PeaksToHarmspect(peaks, wdata.pitch<<4, hspect[hswitch], 1);
 
 			SetBreath();
 		}
@@ -1211,11 +1248,12 @@ int Wavegen()
 				for(pk=wvoice->n_harmonic_peaks+1; pk<N_PEAKS; pk++)
 				{
 					// find the nearest harmonic for HF peaks where we don't use shape
-					peak_harmonic[pk] = peaks[pk].freq / (pitch*16);
+					peak_harmonic[pk] = peaks[pk].freq / (wdata.pitch*16);
 				}
 
 				// adjust amplitude to compensate for fewer harmonics at higher pitch
-				amplitude2 = (amplitude * pitch)/(100 << 11);
+//				amplitude2 = (wdata.amplitude * wdata.pitch)/(100 << 11);
+				amplitude2 = (wdata.amplitude * (wdata.pitch >> 8) * wdata.amplitude_fmt)/(10000 << 3);
 
 				if(glottal_flag > 0)
 				{
@@ -1248,9 +1286,9 @@ int Wavegen()
 					// amplitude envelope is only used for creaky voice effect on certain vowels/tones
 					if((ix = amp_ix>>8) > 127) ix = 127;
 					amp = amplitude_env[ix];
-					amplitude2 = (amplitude2 * amp)/255;
-					if(amp < 255)
-						modulation_type = 7;
+					amplitude2 = (amplitude2 * amp)/128;
+//					if(amp < 255)
+//						modulation_type = 7;
 				}
 
 				// introduce roughness into the sound by reducing the amplitude of
@@ -1337,22 +1375,25 @@ int Wavegen()
 
 		// mix with sampled wave if required
 		z2 = 0;
-		if(mix_wavefile_ix < n_mix_wavefile)
+		if(wdata.mix_wavefile_ix < wdata.n_mix_wavefile)
 		{
-			if(mix_wave_scale == 0)
+			if(wdata.mix_wave_scale == 0)
 			{
 				// a 16 bit sample
-				c = mix_wavefile[mix_wavefile_ix+1];
-				sample = mix_wavefile[mix_wavefile_ix] + (c * 256);
-				mix_wavefile_ix += 2;
+				c = wdata.mix_wavefile[wdata.mix_wavefile_ix+wdata.mix_wavefile_offset+1];
+				sample = wdata.mix_wavefile[wdata.mix_wavefile_ix+wdata.mix_wavefile_offset] + (c * 256);
+				wdata.mix_wavefile_ix += 2;
 			}
 			else
 			{
 				// a 8 bit sample, scaled
-				sample = (signed char)mix_wavefile[mix_wavefile_ix++] * mix_wave_scale;
+				sample = (signed char)wdata.mix_wavefile[wdata.mix_wavefile_offset+wdata.mix_wavefile_ix++] * wdata.mix_wave_scale;
 			}
-			z2 = (sample * amplitude_v) >> 10;
-			z2 = (z2 * mix_wave_amp)/32;
+			z2 = (sample * wdata.amplitude_v) >> 10;
+			z2 = (z2 * wdata.mix_wave_amp)/32;
+
+			if((wdata.mix_wavefile_ix + wdata.mix_wavefile_offset) >= wdata.mix_wavefile_max)  // reached the end of available WAV data
+				wdata.mix_wavefile_offset -= (wdata.mix_wavefile_max*3)/4;
 		}
 
 		z1 = z2 + (((total>>8) * amplitude2) >> 13);
@@ -1397,11 +1438,12 @@ static int PlaySilence(int length, int resume)
 	static int n_samples;
 	int value=0;
 
-	if(length == 0)
-		return(0);
-
 	nsamples = 0;
 	samplecount = 0;
+	wavephase = 0x7fffffff;
+
+	if(length == 0)
+		return(0);
 
 	if(resume==0)
 		n_samples = length;
@@ -1498,14 +1540,32 @@ static int SetWithRange0(int value, int max)
 }
 
 
+static void SetPitchFormants()
+{//===========================
+	int ix;
+	int factor;
+	int pitch_value;
+
+	// adjust formants to give better results for a different voice pitch
+	if((pitch_value = embedded_value[EMBED_P]) > MAX_PITCH_VALUE)
+		pitch_value = MAX_PITCH_VALUE;
+
+	factor = 256 + (25 * (pitch_value - 50))/50;
+	for(ix=0; ix<=5; ix++)
+	{
+		wvoice->freq[ix] = (wvoice->freq2[ix] * factor)/256;
+	}
+	factor = embedded_value[EMBED_T]*3;
+	wvoice->height[0] = (wvoice->height2[0] * (256 - factor*2))/256;
+	wvoice->height[1] = (wvoice->height2[1] * (256 - factor))/256;
+}
+
+
 void SetEmbedded(int control, int value)
 {//=====================================
 	// there was an embedded command in the text at this point
 	int sign=0;
 	int command;
-	int ix;
-	int factor;
-	int pitch_value;
 
 	command = control & 0x1f;
 	if((control & 0x60) == 0x60)
@@ -1528,25 +1588,14 @@ void SetEmbedded(int control, int value)
 	case EMBED_T:
 		WavegenSetEcho();   // and drop through to case P
 	case EMBED_P:
-		// adjust formants to give better results for a different voice pitch
-		if((pitch_value = embedded_value[EMBED_P]) > MAX_PITCH_VALUE)
-			pitch_value = MAX_PITCH_VALUE;
-
-		factor = 256 + (25 * (pitch_value - 50))/50;
-		for(ix=0; ix<=5; ix++)
-		{
-			wvoice->freq[ix] = (wvoice->freq2[ix] * factor)/256;
-		}
-		factor = embedded_value[EMBED_T]*3;
-		wvoice->height[0] = (wvoice->height2[0] * (256 - factor*2))/256;
-		wvoice->height[1] = (wvoice->height2[1] * (256 - factor))/256;
+		SetPitchFormants();
 		break;
 
 	case EMBED_A:  // amplitude
 		general_amplitude = GetAmplitude();
 		break;
 
-	case EMBED_F:   // emphasiis
+	case EMBED_F:   // emphasis
 		general_amplitude = GetAmplitude();
 		break;
 
@@ -1576,6 +1625,9 @@ void WavegenSetVoice(voice_t *v)
 		option_harmonic1 = 6;
 	}
 	WavegenSetEcho();
+	SetPitchFormants();
+	MarkerEvent(espeakEVENT_SAMPLERATE,0,wvoice->samplerate,out_ptr);
+//	WVoiceChanged(wvoice);
 }
 
 
@@ -1587,8 +1639,8 @@ static void SetAmplitude(int length, unsigned char *amp_env, int value)
 	else
 		amp_inc = (256 * ENV_LEN * STEPSIZE)/length;
 
-	amplitude = (value * general_amplitude)/16;
-	amplitude_v = (amplitude * wvoice->consonant_ampv * 15)/100;           // for wave mixed with voiced sounds
+	wdata.amplitude = (value * general_amplitude)/16;
+	wdata.amplitude_v = (wdata.amplitude * wvoice->consonant_ampv * 15)/100;           // for wave mixed with voiced sounds
 
 	amplitude_env = amp_env;
 }
@@ -1620,8 +1672,8 @@ void SetPitch2(voice_t *voice, int pitch1, int pitch2, int *pitch_base, int *pit
 	// compensate for change in pitch when the range is narrowed or widened
 	base -= (range - voice->pitch_range)*18;
 
-	*pitch_base = base + (pitch1 * range);
-	*pitch_range = base + (pitch2 * range) - *pitch_base;
+	*pitch_base = base + (pitch1 * range)/2;
+	*pitch_range = base + (pitch2 * range)/2 - *pitch_base;
 }
 
 
@@ -1641,18 +1693,18 @@ if(option_log_frames)
 	}
 }
 #endif
-	if((pitch_env = env)==NULL)
-		pitch_env = env_fall;  // default
+	if((wdata.pitch_env = env)==NULL)
+		wdata.pitch_env = env_fall;  // default
 
-	pitch_ix = 0;
+	wdata.pitch_ix = 0;
 	if(length==0)
-		pitch_inc = 0;
+		wdata.pitch_inc = 0;
 	else
-		pitch_inc = (256 * ENV_LEN * STEPSIZE)/length;
+		wdata.pitch_inc = (256 * ENV_LEN * STEPSIZE)/length;
 
-	SetPitch2(wvoice, pitch1, pitch2, &pitch_base, &pitch_range);
+	SetPitch2(wvoice, pitch1, pitch2, &wdata.pitch_base, &wdata.pitch_range);
 	// set initial pitch
-	pitch = ((pitch_env[0]*pitch_range)>>8) + pitch_base;   // Hz << 12
+	wdata.pitch = ((wdata.pitch_env[0] * wdata.pitch_range) >>8) + wdata.pitch_base;   // Hz << 12
 
 	flutter_amp = wvoice->flutter;
 
@@ -1662,15 +1714,14 @@ if(option_log_frames)
 
 
 
-void SetSynth(int length, int modn, frame_t *fr1, frame_t *fr2)
-{//============================================================
+void SetSynth(int length, int modn, frame_t *fr1, frame_t *fr2, voice_t *v)
+{//========================================================================
 	int ix;
 	DOUBLEX next;
 	int length2;
 	int length4;
 	int qix;
 	int cmd;
-	voice_t *v;
 	static int glottal_reduce_tab1[4] = {0x30, 0x30, 0x40, 0x50};  // vowel before [?], amp * 1/256
 //	static int glottal_reduce_tab1[4] = {0x30, 0x40, 0x50, 0x60};  // vowel before [?], amp * 1/256
 	static int glottal_reduce_tab2[4] = {0x90, 0xa0, 0xb0, 0xc0};  // vowel after [?], amp * 1/256
@@ -1724,8 +1775,6 @@ if(option_log_frames)
 			break;   // next is not from spectrum, so continue until end of wave cycle
 	}
 
-	v = wvoice;
-
 	// round the length to a multiple of the stepsize
 	length2 = (length + STEPSIZE/2) & ~0x3f;
 	if(length2 == 0)
@@ -1736,29 +1785,43 @@ if(option_log_frames)
 	nsamples += length2;
 
 	length4 = length2/4;
-	for(ix=0; ix<N_PEAKS; ix++)
+
+	peaks[7].freq = (7800  * v->freq[7] + v->freqadd[7]*256) << 8;
+	peaks[8].freq = (9000  * v->freq[8] + v->freqadd[8]*256) << 8;
+
+	for(ix=0; ix < 8; ix++)
 	{
-		peaks[ix].freq1 = (fr1->ffreq[ix] * v->freq[ix] + v->freqadd[ix]*256) << 8;
-		peaks[ix].freq = int(peaks[ix].freq1);
-		next = (fr2->ffreq[ix] * v->freq[ix] + v->freqadd[ix]*256) << 8;
-		peaks[ix].freq_inc =  ((next - peaks[ix].freq1) * (STEPSIZE/4)) / length4;  // lower headroom for fixed point math
+		if(ix < 7)
+		{
+			peaks[ix].freq1 = (fr1->ffreq[ix] * v->freq[ix] + v->freqadd[ix]*256) << 8;
+			peaks[ix].freq = int(peaks[ix].freq1);
+			next = (fr2->ffreq[ix] * v->freq[ix] + v->freqadd[ix]*256) << 8;
+			peaks[ix].freq_inc =  ((next - peaks[ix].freq1) * (STEPSIZE/4)) / length4;  // lower headroom for fixed point math
+		}
 
 		peaks[ix].height1 = (fr1->fheight[ix] * v->height[ix]) << 6;
 		peaks[ix].height = int(peaks[ix].height1);
 		next = (fr2->fheight[ix] * v->height[ix]) << 6;
 		peaks[ix].height_inc =  ((next - peaks[ix].height1) * STEPSIZE) / length2;
 
-		if(ix <= wvoice->n_harmonic_peaks)
+		if((ix <= 5) && (ix <= wvoice->n_harmonic_peaks))
 		{
 			peaks[ix].left1 = (fr1->fwidth[ix] * v->width[ix]) << 10;
 			peaks[ix].left = int(peaks[ix].left1);
 			next = (fr2->fwidth[ix] * v->width[ix]) << 10;
 			peaks[ix].left_inc =  ((next - peaks[ix].left1) * STEPSIZE) / length2;
 
-			peaks[ix].right1 = (fr1->fright[ix] * v->width[ix]) << 10;
-			peaks[ix].right = int(peaks[ix].right1);
-			next = (fr2->fright[ix] * v->width[ix]) << 10;
-			peaks[ix].right_inc = ((next - peaks[ix].right1) * STEPSIZE) / length2;
+			if(ix < 3)
+			{
+				peaks[ix].right1 = (fr1->fright[ix] * v->width[ix]) << 10;
+				peaks[ix].right = int(peaks[ix].right1);
+				next = (fr2->fright[ix] * v->width[ix]) << 10;
+				peaks[ix].right_inc = ((next - peaks[ix].right1) * STEPSIZE) / length2;
+			}
+			else
+			{
+				peaks[ix].right = peaks[ix].left;
+			}
 		}
 	}
 }  // end of SetSynth
@@ -1767,7 +1830,7 @@ if(option_log_frames)
 static int Wavegen2(int length, int modulation, int resume, frame_t *fr1, frame_t *fr2)
 {//====================================================================================
 	if(resume==0)
-		SetSynth(length,modulation,fr1,fr2);
+		SetSynth(length, modulation, fr1, fr2, wvoice);
 
 	return(Wavegen());
 }
@@ -1786,8 +1849,7 @@ void Write4Bytes(FILE *f, int value)
 
 
 
-
-int WavegenFill(int fill_zeros)
+int WavegenFill2(int fill_zeros)
 {//============================
 // Pick up next wavegen commands from the queue
 // return: 0  output buffer has been filled
@@ -1799,11 +1861,6 @@ int WavegenFill(int fill_zeros)
 	static int resume=0;
 	static int echo_complete=0;
 
-#ifdef TEST_MBROLA
-	if(mbrola_name[0] != 0)
-		return(MbrolaFill(fill_zeros));
-#endif
-
 	while(out_ptr < out_end)
 	{
 		if(WcmdqUsed() <= 0)
@@ -1839,35 +1896,51 @@ int WavegenFill(int fill_zeros)
 			{
 				echo_complete -= length;
 			}
-			n_mix_wavefile = 0;
+			wdata.n_mix_wavefile = 0;
+			wdata.amplitude_fmt = 100;
+			KlattReset(1);
 			result = PlaySilence(length,resume);
 			break;
 
 		case WCMD_WAVE:
 			echo_complete = echo_length;
-			n_mix_wavefile = 0;
+			wdata.n_mix_wavefile = 0;
+			KlattReset(1);
 			result = PlayWave(length,resume,(unsigned char*)q[2], q[3] & 0xff, q[3] >> 8);
 			break;
 
 		case WCMD_WAVE2:
 			// wave file to be played at the same time as synthesis
-			mix_wave_amp = q[3] >> 8;
-			mix_wave_scale = q[3] & 0xff;
-			if(mix_wave_scale == 0)
-				n_mix_wavefile = length*2;
-			else
-				n_mix_wavefile = length;
-			mix_wavefile_ix = 0;
-			mix_wavefile = (unsigned char *)q[2];
+			wdata.mix_wave_amp = q[3] >> 8;
+			wdata.mix_wave_scale = q[3] & 0xff;
+			wdata.n_mix_wavefile = (length & 0xffff);
+			wdata.mix_wavefile_max = (length >> 16) & 0xffff;
+			if(wdata.mix_wave_scale == 0)
+			{
+				wdata.n_mix_wavefile *= 2;
+				wdata.mix_wavefile_max *= 2;
+			}
+			wdata.mix_wavefile_ix = 0;
+			wdata.mix_wavefile_offset = 0;
+			wdata.mix_wavefile = (unsigned char *)q[2];
 			break;
 
 		case WCMD_SPECT2:   // as WCMD_SPECT but stop any concurrent wave file
-			n_mix_wavefile = 0;   // ... and drop through to WCMD_SPECT case
+			wdata.n_mix_wavefile = 0;   // ... and drop through to WCMD_SPECT case
 		case WCMD_SPECT:
 			echo_complete = echo_length;
 			result = Wavegen2(length & 0xffff,q[1] >> 16,resume,(frame_t *)q[2],(frame_t *)q[3]);
 			break;
 
+#ifdef INCLUDE_KLATT
+		case WCMD_KLATT2:   // as WCMD_SPECT but stop any concurrent wave file
+			wdata.n_mix_wavefile = 0;   // ... and drop through to WCMD_SPECT case
+		case WCMD_KLATT:
+			echo_complete = echo_length;
+			result = Wavegen_Klatt2(length & 0xffff,q[1] >> 16,resume,(frame_t *)q[2],(frame_t *)q[3]);
+			break;
+#endif
+
 		case WCMD_MARKER:
 			MarkerEvent(q[1],q[2],q[3],out_ptr);
 #ifdef LOG_FRAMES
@@ -1891,6 +1964,19 @@ int WavegenFill(int fill_zeros)
 		case WCMD_EMBEDDED:
 			SetEmbedded(q[1],q[2]);
 			break;
+
+		case WCMD_MBROLA_DATA:
+			result = MbrolaFill(length, resume, (general_amplitude * wvoice->voicing)/64);
+			break;
+
+		case WCMD_FMT_AMPLITUDE:
+			if((wdata.amplitude_fmt = q[1]) == 0)
+				wdata.amplitude_fmt = 100;  // percentage, but value=0 means 100%
+			break;
+
+		case WCMD_SONIC_SPEED:
+			sonicSpeed = (double)q[1] / 1024;
+			break;
 		}
 
 		if(result==0)
@@ -1905,6 +1991,59 @@ int WavegenFill(int fill_zeros)
 	}
 
 	return(0);
+}  // end of WavegenFill2
+
+
+/* Speed up the audio samples with libsonic. */
+static int SpeedUp(short *outbuf, int length_in, int length_out, int end_of_text)
+{//==============================================================================
+	if(length_in >0)
+	{
+		if(sonicSpeedupStream == NULL)
+		{
+			sonicSpeedupStream = sonicCreateStream(22050, 1);
+		}
+		if(sonicGetSpeed(sonicSpeedupStream) != sonicSpeed)
+		{
+		        sonicSetSpeed(sonicSpeedupStream, sonicSpeed);
+		}
+	
+		sonicWriteShortToStream(sonicSpeedupStream, outbuf, length_in);
+	}
+
+	if(sonicSpeedupStream == NULL)
+		return(0);
+
+	if(end_of_text)
+	{
+		sonicFlushStream(sonicSpeedupStream);
+	}
+	return sonicReadShortFromStream(sonicSpeedupStream, outbuf, length_out);
+}  // end of SpeedUp
+
+
+/* Call WavegenFill2, and then speed up the output samples. */
+int WavegenFill(int fill_zeros)
+{//============================
+	int finished;
+	unsigned char *p_start;
+	int length;
+	int max_length;
+
+	p_start = out_ptr;
+
+	// fill_zeros is ignored. It is now done in the portaudio callback
+	finished = WavegenFill2(0);
+
+	if(sonicSpeed > 1.0)
+	{
+		max_length = (out_end - p_start);
+		length =  2*SpeedUp((short *)p_start, (out_ptr-p_start)/2, max_length/2, finished);
+		out_ptr = p_start + length;
+
+		if(length >= max_length)
+			finished = 0;   // there may be more data to flush
+	}
+	return finished;
 }  // end of WavegenFill
 
-
